@@ -1,4 +1,4 @@
-""" doit tasks for ipyforcegraph
+"""doit tasks for ``ipyforcegraph``
 
     Generally, you'll just want to `doit`.
 
@@ -32,6 +32,7 @@ from scripts import reporter
 from scripts import utils as U
 
 os.environ.update(
+    BLACK_CACHE_DIR=str(P.BUILD / ".black"),
     MAMBA_NO_BANNER="1",
     NODE_OPTS="--max-old-space-size=4096",
     PIP_DISABLE_PIP_VERSION_CHECK="1",
@@ -157,7 +158,7 @@ def _ok(task, ok):
         lambda: [ok.exists() and ok.unlink(), True][-1],
         *task["actions"],
         lambda: [
-            ok.parent.mkdir(exist_ok=True),
+            ok.parent.mkdir(exist_ok=True, parents=True),
             ok.write_text("ok", encoding="utf-8"),
             True,
         ][-1],
@@ -525,6 +526,15 @@ def task_test():
     for nb in P.EXAMPLE_IPYNB:
         yield _nb_test(nb)
 
+    for robot_template in P.ATEST.rglob("*.j2"):
+        robot_out = robot_template.parent / robot_template.name.replace(".j2", "")
+        yield dict(
+            name=f"atest:template:{robot_template.relative_to(P.ATEST)}",
+            actions=[(U.template_one, [robot_template, robot_out])],
+            file_dep=[robot_template],
+            targets=[robot_out],
+        )
+
     def _pabot_logs():
         for robot_out in sorted(P.ATEST_OUT.rglob("robot_*.out")):
             print(f"\n[{robot_out.relative_to(P.ROOT)}]")
@@ -553,28 +563,28 @@ def task_lint():
     if P.TESTING_IN_CI:
         return
 
-    def _ssort():
-        rc = subprocess.call(list(map(str, [*P.IN_ENV, "ssort", *P.ALL_PY])))
-        if rc != 0:
-            print(">>> Don't worry about failing `ssort`")
-
-    if "py_3.7" in str(P.ENV):
-        ssort = lambda: None
-    else:
-        ssort = [*P.IN_ENV, "ssort", *P.ALL_PY]
+    yield _ok(
+        dict(
+            name="pyproject",
+            file_dep=[P.PY_PROJ, P.HISTORY],
+            actions=[[*P.IN_ENV, "pyproject-fmt", P.PY_PROJ]],
+        ),
+        P.OK_PYPROJ_FMT,
+    )
 
     yield _ok(
         dict(
             name="black",
             file_dep=[*P.ALL_PY, P.HISTORY],
             actions=[
-                ssort,
+                [*P.IN_ENV, "ssort", *P.ALL_PY],
                 [*P.IN_ENV, "isort", "--quiet", "--ac", *P.ALL_PY],
                 [*P.IN_ENV, "black", "--quiet", *P.ALL_PY],
             ],
         ),
         P.OK_BLACK,
     )
+
     yield _ok(
         dict(
             name="pyflakes",
@@ -583,6 +593,16 @@ def task_lint():
         ),
         P.OK_PYFLAKES,
     )
+
+    yield _ok(
+        dict(
+            name="mypy",
+            file_dep=[*P.ALL_PY, P.OK_BLACK],
+            actions=[[*P.IN_ENV, "mypy", *P.ALL_PY_SRC]],
+        ),
+        P.OK_MYPY,
+    )
+
     yield _ok(
         dict(
             name="prettier",
@@ -613,8 +633,18 @@ def task_lint():
                 name=f"nblint:{nb.name}".replace(" ", "_").replace(".ipynb", ""),
                 file_dep=[P.YARN_INTEGRITY, nb, P.HISTORY, P.OK_BLACK],
                 actions=[
-                    [*P.IN_ENV, *P.PYM, "scripts.nblint", nb],
-                    [*P.IN_ENV, "black", "--quiet", nb],
+                    [*P.IN_ENV, "nbstripout", nb],
+                    (U.notebook_lint, [nb]),
+                    [
+                        *P.IN_ENV,
+                        "jupyter-nbconvert",
+                        "--log-level=WARN",
+                        "--to=notebook",
+                        "--inplace",
+                        "--output",
+                        nb,
+                        nb,
+                    ],
                 ],
             ),
             P.OK_NBLINT[nb.name],
@@ -668,10 +698,11 @@ def task_lint():
             actions=[_echo_ok("all ok")],
             file_dep=[
                 P.OK_BLACK,
+                P.OK_INDEX,
                 P.OK_PRETTIER,
                 P.OK_PYFLAKES,
+                P.OK_PYPROJ_FMT,
                 P.OK_ROBOT_LINT,
-                P.OK_INDEX,
             ],
         ),
         P.OK_LINT,
@@ -780,7 +811,14 @@ def task_watch_docs():
         file_dep=[P.DOCS_BUILDINFO, *P.ALL_MD, P.OK_PIP_INSTALL],
         actions=[
             LongRunning(
-                [*P.IN_ENV, "sphinx-autobuild", P.DOCS, P.DOCS_BUILD], shell=False
+                [
+                    *P.IN_ENV,
+                    "sphinx-autobuild",
+                    f"--watch={P.PY_SRC}",
+                    P.DOCS,
+                    P.DOCS_BUILD,
+                ],
+                shell=False,
             )
         ],
     )
@@ -788,33 +826,72 @@ def task_watch_docs():
 
 def _make_spellcheck(dep, html):
     ok = P.BUILD / "spell" / f"{dep.relative_to(html)}.ok"
+    fail_path = P.BUILD / "spell" / f"{dep.relative_to(html)}.fail"
+
+    def spell():
+        if fail_path.exists():
+            fail_path.unlink()
+        fails = sorted(
+            set(
+                subprocess.check_output(
+                    [
+                        *P.IN_ENV,
+                        "hunspell",
+                        "-l",
+                        "-d",
+                        "en_US,en-GB",
+                        "-p",
+                        P.DICTIONARY,
+                        "-H",
+                        dep,
+                    ]
+                )
+                .decode("utf-8")
+                .strip()
+                .splitlines()
+            )
+        )
+
+        if fails:
+            fail_str = "\n".join(fails)
+            print("Unrecognized words in", dep)
+            print(fail_str)
+            fail_path.parent.mkdir(exist_ok=True, parents=True)
+            fail_path.write_text(fail_str, encoding="utf-8")
+
     return _ok(
         dict(
             name=f"""spell:{dep.relative_to(html)}""".replace("/", "_"),
             doc=f"check spelling in {dep.relative_to(html)}",
             file_dep=[dep, P.DICTIONARY],
-            actions=[
-                [
-                    *P.IN_ENV,
-                    "hunspell",
-                    "-l",
-                    "-d",
-                    "en_US,en-GB",
-                    "-p",
-                    P.DICTIONARY,
-                    "-H",
-                    dep,
-                ]
-            ],
+            actions=[spell],
         ),
         ok,
     )
 
 
+def _all_spell():
+    all_fail = []
+    for path in sorted(P.ALL_SPELL.parent.rglob("*.fail")):
+        if path == P.ALL_SPELL:
+            continue
+        all_fail += path.read_text(encoding="utf-8").strip().splitlines()
+
+    all_fail_str = "\n".join(sorted(set(all_fail)))
+
+    if all_fail_str:
+        print("ALL Unrecognized words")
+        print(all_fail_str)
+
+    P.ALL_SPELL.write_text(all_fail_str, encoding="utf-8")
+
+    return len(all_fail) == 0
+
+
 @create_after(executed="docs", target_regex=r"build/docs/html/.*\.html")
 def task_checkdocs():
     """check spelling and links of build docs HTML."""
-    no_check = ["htmlcov", "pytest", "_static"]
+    no_check = ["htmlcov", "pytest", "_static", "genindex"]
     html = P.DOCS_BUILD / "html"
     file_dep = sorted(
         {
@@ -852,5 +929,16 @@ def task_checkdocs():
         P.OK_LINKS,
     )
 
+    spell_tasks = []
+
     for dep in file_dep:
-        yield _make_spellcheck(dep, html)
+        task = _make_spellcheck(dep, html)
+        spell_tasks += [f"""checkdocs:{task["name"]}"""]
+        yield task
+
+    yield dict(
+        name="spell:ALL",
+        task_dep=spell_tasks,
+        actions=[_all_spell],
+        targets=[P.ALL_SPELL],
+    )
