@@ -9,11 +9,11 @@ except Exception:  # pragma: no cover
     from importlib_metadata import version
 
 
-def test_meta():
+def test_meta() -> None:
     assert hasattr(ipyforcegraph, "__version__")
     assert ipyforcegraph.__version__ == version("ipyforcegraph")
 
 
-def test_labext():
+def test_labext() -> None:
     extensions = ipyforcegraph._jupyter_labextension_paths()
     assert len(extensions) == 1, "unexpected number of extensions"
